/****************************************************************************
** Meta object code from reading C++ file 'networkmanagerprocesser.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.2)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../impl/networkmanager/networkmanagerprocesser.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'networkmanagerprocesser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.2. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN3dde7network23NetworkManagerProcesserE_t {};
} // unnamed namespace

template <> constexpr inline auto dde::network::NetworkManagerProcesser::qt_create_metaobjectdata<qt_meta_tag_ZN3dde7network23NetworkManagerProcesserE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "dde::network::NetworkManagerProcesser",
        "onDeviceAdded",
        "",
        "uni",
        "onDeviceRemove",
        "onConnectivityChanged",
        "NetworkManager::Connectivity",
        "conntity",
        "onDevicesChanged",
        "QList<QDBusObjectPath>",
        "devices",
        "checkConnectivityFinished",
        "onUpdateNetworkDetail"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'onDeviceAdded'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'onDeviceRemove'
        QtMocHelpers::SlotData<void(const QString &)>(4, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'onConnectivityChanged'
        QtMocHelpers::SlotData<void(NetworkManager::Connectivity)>(5, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Slot 'onDevicesChanged'
        QtMocHelpers::SlotData<void(const QList<QDBusObjectPath> &)>(8, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'checkConnectivityFinished'
        QtMocHelpers::SlotData<void(quint32)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::UInt, 7 },
        }}),
        // Slot 'onUpdateNetworkDetail'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NetworkManagerProcesser, qt_meta_tag_ZN3dde7network23NetworkManagerProcesserE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject dde::network::NetworkManagerProcesser::staticMetaObject = { {
    QMetaObject::SuperData::link<NetworkProcesser::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dde7network23NetworkManagerProcesserE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dde7network23NetworkManagerProcesserE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN3dde7network23NetworkManagerProcesserE_t>.metaTypes,
    nullptr
} };

void dde::network::NetworkManagerProcesser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NetworkManagerProcesser *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->onDeviceAdded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->onDeviceRemove((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->onConnectivityChanged((*reinterpret_cast<std::add_pointer_t<NetworkManager::Connectivity>>(_a[1]))); break;
        case 3: _t->onDevicesChanged((*reinterpret_cast<std::add_pointer_t<QList<QDBusObjectPath>>>(_a[1]))); break;
        case 4: _t->checkConnectivityFinished((*reinterpret_cast<std::add_pointer_t<quint32>>(_a[1]))); break;
        case 5: _t->onUpdateNetworkDetail(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<QDBusObjectPath> >(); break;
            }
            break;
        }
    }
}

const QMetaObject *dde::network::NetworkManagerProcesser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *dde::network::NetworkManagerProcesser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN3dde7network23NetworkManagerProcesserE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "ProcesserInterface"))
        return static_cast< ProcesserInterface*>(this);
    return NetworkProcesser::qt_metacast(_clname);
}

int dde::network::NetworkManagerProcesser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = NetworkProcesser::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}
QT_WARNING_POP
